@@ -3,7 +3,9 @@ import os
 import sys
 
 if __name__ == "__main__":
-    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "core.settings")
+    sys.path.append(os.path.dirname(os.path.dirname(
+        os.path.abspath(__file__))))
+    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "shiori.core.settings")
 
     from django.core.management import execute_from_command_line
 
